############################################################################
#    Copyright (C) 2008 by David Palacio                                   #
#    dpalacio@uninorte.edu.co                                              #
#                                                                          #
#    This program is free software; you can redistribute it and#or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 3 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          #
############################################################################

require 'korundum4'
require 'okularpart'

module Reader
	class MainWindow < KDE::XmlGuiWindow
		
		slots :open, 'open(const KUrl&)', :open_next, :open_previous
		
		def initialize
			super # nil, 0 para empanadear el constructor de KParts::MainWindow
			@part = OkularPart.create self
			set_central_widget @part.widget
			
			setup_actions
		end
		
		def setup_actions
			KDE::StandardAction.open self, SLOT('open()'), action_collection
			@recent_action = KDE::StandardAction.open_recent self, SLOT('open(const KUrl&)'), action_collection
			@recent_action.load_entries( KDE::Global.config.group 'Recent Files' )
			next_action = KDE::Action.new self
			next_action.text = i18n 'Open Next'
			next_action.icon = KDE::Icon.new 'go-next'
			next_action.shortcut = KDE::Shortcut.new( 'CTRL+N; CTRL+Right' )
			connect next_action, SIGNAL('triggered()'), self, SLOT('open_next()')
			action_collection.add_action 'next', next_action
			previous_action = KDE::Action.new self
			previous_action.text = i18n 'Open Previous'
			previous_action.icon = KDE::Icon.new 'go-previous'
			previous_action.shortcut = KDE::Shortcut.new( 'CTRL+P; CTRL+Left' )
			connect previous_action, SIGNAL('triggered()'), self, SLOT('open_previous()')
			action_collection.add_action 'previous', previous_action
# 			KDE::StandardAction.close self, SLOT('close()'), action_collection
			KDE::StandardAction.quit $kapp, SLOT('quit()'), action_collection
			setupGUI KDE::XmlGuiWindow::Default, 'mainwindow.rc'
		end
		
		def open( path = nil )
			path ||= KDE::FileDialog.get_open_url KDE::Url.new( 'kfiledialog:///omaque' ), '*.pdf *.zip *.cbz *.rar *.cbr', self, 'Open'
			return if path.empty?
			@url = @part.openUrl( path ) ? path : nil
			if @url
                                @recent_action.add_url( @url )
				puts @url.inspect
				puts @url.path
				set_caption @url.file_name
			end
			@url
		end
		
		def local_open?
			@url && !@url.empty? && @url.isLocalFile # !! no funciona «local_file?»
		end
		
		def dir_entries( path, order = Qt::Dir::Name )
			Qt::Dir.new( path ).entry_list [ '*.zip', '*.cbz' ], Qt::Dir::Files | Qt::Dir::Readable, Qt::Dir::Name | Qt::Dir::IgnoreCase | order
		end
		
		def get_next( path, entries, name )
			next_file = nil
			entries.inject do |previous, current|
				if previous == name
					next_file = current
					break
				end
				current
			end
			return KDE::Url.new unless next_file #ya estamos llamando open así que mejor enviemos un archivo inválido
			return KDE::Url.new( path + '/' + next_file )
		end
		
		def open_next
			return unless local_open?
			path = @url.up_url.path
			entries = dir_entries path
			open get_next( path, entries, @url.file_name )
		end
		
		def open_previous
			return unless local_open?
			path = @url.up_url.path
			entries = dir_entries( path, Qt::Dir::Reversed )
			open get_next( path, entries, @url.file_name )
		end
		
		def close
			STDERR.puts 'close: unsupported'
# 			Qt::MetaObject.invoke_method @part, 'closeUrl' #la parte dice que no se puede cerrar fuera de Okular o.O
		end
		
		def queryExit
			group = KDE::Global.config.group 'Recent Files'
			@recent_action.save_entries group
			group.sync #Guarda, que estamos saliendo
			true
		end
		
	end
end

about = KDE::AboutData.new(
  'omaque', # app internal name
  'omaque', # translation catalog name
  KDE.ki18n( 'Omaque' ), # name to be displayed
  'alfa', #version
  KDE.ki18n( 'An extra comic reader based on Okular' ), #description
  KDE::AboutData::License_GPL_V3, #license
  KDE.ki18n( '(c) 2008 David Palacio' ) # copyright statement
  ) #AboutData
KDE::CmdLineArgs.init([], about)
KDE::Application.new
main = Reader::MainWindow.new
main.show
$kapp.exec
